# the plugin code was generated by this script
# you should not change the plugin code directly, but this script
# note: this script only creates objects in your workspace,
# *EXCEPT* for the last call, see below.

require(rkwarddev)

local({
# set the output directory to overwrite the actual plugin
output.dir <- tempdir()
overwrite <- TRUE

about.info <- rk.XML.about(
	name="rk.ClusterAnalysis",
	author=c(
		person(given="Meik", family="Michalke",
			email="meik.michalke@hhu.de", role=c("aut","cre"))),
	about=list(desc="RKWard GUI to conduct k-means, model based and hierarchical cluster analyses",
		version="0.01-0", url="http://rkward.sf.net"),
	dependencies=list(rkward.min="0.5.6"),
	package=list(c(name="mclust"))
	)

############
## prepare data
############
clust.pre.omitNA <- rk.XML.cbox("Remove missing values", value="true", chk=TRUE)
clust.pre.scale <- rk.XML.cbox("Stadardize values", value="true")
clust.pre.frame <- rk.XML.frame(
	rk.XML.row(
		rk.XML.col(clust.pre.omitNA),
		rk.XML.col(clust.pre.scale)
	),
	label="Data preparation")

############
## k-means
############
clust.k.var.select <- rk.XML.varselector(label="Select data")
clust.k.var.data <- rk.XML.varslot(label="Data (matrix or data.frame)", source=clust.k.var.select, classes=c("data.frame", "matrix"), required=TRUE)

clust.k.spin.numcl <- rk.XML.spinbox(label="Number of clusters to extract", min=2, real=FALSE)

clust.k.drop.meth <- rk.XML.dropdown(label="Algorithm", options=list(
		"Hartigan & Wong"=c(val="Hartigan-Wong", chk=TRUE),
		"Lloyd"=c(val="Lloyd"),
		"Forgy"=c(val="Forgy"),
		"MacQueen"=c(val="MacQueen")
	))

clust.k.spin.maxiter <- rk.XML.spinbox(label="Maximum number of iterations", min=1, initial=10, real=FALSE)
clust.k.spin.nstart <- rk.XML.spinbox(label="Initial random set of centers", min=1, initial=1, real=FALSE)

save.results.k <- rk.XML.saveobj("Save results to workspace", initial="clust.k.result")

# plot results
clust.plotk.cbox.dend <- rk.XML.cbox("Plot results", val="true", chk=TRUE)
clust.plotk.chk.points <- rk.XML.cbox("Plot cluster centers", val="true", chk=TRUE)

clust.plotk.preview <- rk.XML.preview()

tab.k.data <- rk.XML.row(
		clust.k.var.select,
		rk.XML.col(
			clust.k.var.data,
			clust.pre.frame,
			rk.XML.frame(clust.k.spin.numcl),
			rk.XML.stretch(),
			rk.XML.frame(
				clust.k.drop.meth,
				rk.XML.row(
					rk.XML.col(clust.k.spin.maxiter),
					rk.XML.col(clust.k.spin.nstart)
				),
				clust.plotk.cbox.dend,
				label="Advanced options"),
			save.results.k
		)
	)

tab.k.plot <- rk.XML.row(
		rk.XML.col(
			clust.plotk.chk.points,
			rk.XML.stretch(),
			clust.plotk.preview)
	)

clust.k.full.dialog <- rk.XML.dialog(
	rk.XML.tabbook(tabs=list(
		"Data"=tab.k.data,
		"Plot"=tab.k.plot
	)),
	label="Cluster analysis: K-means partitioning")

lgc.sect.k <- rk.XML.logic(
		rk.XML.connect(governor=clust.plotk.cbox.dend, client=tab.k.plot, set="enabled")
	)

## JavaScript
clust.k.js.calc <- rk.paste.JS(
	ite(clust.pre.omitNA, echo("\t# Listwise removal of missings\n\t",
		clust.k.var.data, " <- na.omit(", clust.k.var.data, ")\n")),
	ite(clust.pre.scale, echo("\t# Standardizing values\n\t",
		clust.k.var.data, " <- scale(", clust.k.var.data, ")\n")),
	echo("\tclust.k.result <- kmeans("),
	ite(clust.k.var.data, echo("\n\t\tx=", clust.k.var.data)),
	echo(",\n\t\tcenters=", clust.k.spin.numcl),
	ite(id(clust.k.drop.meth, " != \"Hartigan-Wong\""), echo(",\n\t\talgorithm=\"", clust.k.drop.meth,"\"")),
	ite(id(clust.k.spin.maxiter, " != 10"), echo(",\n\t\titer.max=", clust.k.spin.maxiter)),
	ite(id(clust.k.spin.nstart, " != 1"), echo(",\n\t\tnstart=", clust.k.spin.nstart)),
	echo("\n\t)\n\n")
)

clust.k.js.plot <- rk.paste.JS(
	ite(clust.plotk.cbox.dend, rk.paste.JS(
			ite("full", echo("rk.graph.on()\n")),
			echo("\ttry({\n\t\tplot(", clust.k.var.data,", col=clust.k.result$cluster, ",
			"main=\"K-means partitioning\", sub=\"Grouped into ", clust.k.spin.numcl, " clusters by the ", clust.k.drop.meth, " algorithm\")"),
			ite(clust.plotk.chk.points,
				echo("\n\t\tpoints(clust.k.result$centers, col=1:", clust.k.spin.numcl, ", pch=8, cex=2)")),
			echo("\n\t})\n"),
			ite("full", echo("rk.graph.off()\n")), level=1),
		echo("\trk.print(clust.k.result)\n"))
)

############
## hierarchical
############
clust.h.var.select <- rk.XML.varselector(label="Select data")
clust.h.var.data <- rk.XML.varslot(label="Data (matrix or data.frame)", source=clust.h.var.select, classes=c("data.frame", "matrix"), required=TRUE)

clust.h.drop.dist <- rk.XML.dropdown(label="Computation method", options=list(
		"Euclidean"=c(val="euclidean", chk=TRUE),
		"Maximum"=c(val="maximum"),
		"Manhattan (city block)"=c(val="manhattan"),
		"Canberra"=c(val="canberra"),
		"Binary"=c(val="binary"),
		"Minkowski"=c(val="minkowski")
	))
clust.h.drop.clst <- rk.XML.dropdown(label="Agglomeration method", options=list(
		"Ward"=c(val="ward", chk=TRUE),
		"Single linkage (nearest neighbor)"=c(val="single"),
		"Complete linkage (furthest neighbor)"=c(val="complete"),
		"Average linkage (between groups linkage)"=c(val="average"),
		"McQuitty"=c(val="mcquitty"),
		"Median clustering"=c(val="median"),
		"Centroid clustering"=c(val="centroid")
	))
clust.h.spin.pwmink <- rk.XML.spinbox(label="Power of Minkowski distance", min=1, initial=2, real=FALSE)

save.results.h <- rk.XML.saveobj("Save results to workspace", initial="clust.h.result")

# dendrogram
clust.dend.cbox.dend <- rk.XML.cbox("Draw dendrogram", val="true", chk=TRUE)
clust.dend.spin.numcl <- rk.XML.spinbox(label="Draw border around clusters (1 for none)", min=1, initial=1, real=FALSE)

clust.dend.preview <- rk.XML.preview()

tab.data <- rk.XML.row(
		clust.h.var.select,
		rk.XML.col(
			clust.h.var.data,
			clust.pre.frame,
			rk.XML.stretch(),
			rk.XML.frame(clust.h.drop.dist, clust.h.spin.pwmink, label="Distance matrix"),
			rk.XML.frame(clust.h.drop.clst, label="Clustering"),
			clust.dend.cbox.dend,
			save.results.h
		)
	)

tab.dendrogram <- rk.XML.row(
		rk.XML.col(
			clust.dend.spin.numcl,
			clust.dend.preview)
	)

clust.h.full.dialog <- rk.XML.dialog(
	rk.XML.tabbook(tabs=list(
		"Data"=tab.data,	
		"Dendrogram"=tab.dendrogram
	)),
	label="Cluster analysis: Hierarchical")

## logic section
	lgc.sect.h <- rk.XML.logic(
		CA.gov.dist <- rk.XML.convert(sources=list(string=clust.h.drop.dist), mode=c(equals="minkowski")),
		rk.XML.connect(governor=CA.gov.dist, client=clust.h.spin.pwmink, set="enabled"),
		rk.XML.connect(governor=clust.dend.cbox.dend, client=tab.dendrogram, set="enabled")
	)

## JavaScript
clust.h.js.calc <- rk.paste.JS(
	ite(clust.pre.omitNA, echo("\t# Listwise removal of missings\n\t",
		clust.h.var.data, " <- na.omit(", clust.h.var.data, ")\n")),
	ite(clust.pre.scale, echo("\t# Standardizing values\n\t",
		clust.h.var.data, " <- scale(", clust.h.var.data, ")\n")),
	echo("\t# Compute distance matrix\n\tclust.h.distances <- dist("),
	ite(clust.h.var.data, echo("\n\t\tx=", clust.h.var.data)),
	echo(",\n\t\tmethod=\"", clust.h.drop.dist, "\""),
	ite(id(clust.h.drop.dist, " == \"minkowski\""), echo(",\n\t\tp=", clust.h.spin.pwmink)),
	echo("\n\t)\n"),
	echo("\t# Hierarchical CA\n\tclust.h.result <- hclust(d=clust.h.distances"),
	echo(",\n\t\tmethod=\"", clust.h.drop.clst, "\""),
	echo("\n\t)\n\n")
)

clust.h.js.dend <- rk.paste.JS(
	ite(clust.dend.cbox.dend, rk.paste.JS(
			ite("full", echo("rk.graph.on()\n")),
			echo("\ttry({\n\t\tplot(clust.h.result)"),
			ite(id(clust.dend.spin.numcl, " > 1"),
				echo("\n\t\trect.hclust(clust.h.result, k=", clust.dend.spin.numcl, ", border=\"red\")")),
			echo("\n\t})\n"),
			ite("full", echo("rk.graph.off()\n")), level=1),
		echo("\trk.print(clust.h.result)\n"))
)

## make a whole component
clust.h.component <- rk.plugin.component("Hierarchical CA",
	xml=list(
		dialog=clust.h.full.dialog,
		logic=lgc.sect.h),
	js=list(
#		require="fcp",
		calculate=clust.h.js.calc,
		doPrintout=clust.h.js.dend
	),
	hierarchy=list("analysis", "Cluster analysis"),
	create=c("xml", "js"))

#############
## model based CA
#############
clust.m.var.select <- rk.XML.varselector(label="Select data")
clust.m.var.data <- rk.XML.varslot(label="Data (matrix or data.frame)", source=clust.m.var.select, classes=c("data.frame", "matrix"), required=TRUE)

clust.m.spin.numcl <- rk.XML.spinbox(label="Max number of clusters to test", min=2, initial=9, real=FALSE)

save.results.m <- rk.XML.saveobj("Save results to workspace", initial="clust.m.result")

# dendrogram
clust.plotm.cbox.plot <- rk.XML.cbox("Plot results", val="true", chk=TRUE)
clust.plotm.type <- rk.XML.radio("Plot type", options=list(
		"BIC"=c(val="BIC", chk=TRUE),
		"Classification"=c(val="classification"),
		"Classification uncertainty"=c(val="uncertainty"),
		"Density"=c(val="density")
	))

clust.plotm.preview <- rk.XML.preview()

tab.m.data <- rk.XML.row(
		clust.m.var.select,
		rk.XML.col(
			clust.m.var.data,
			clust.pre.frame,
			rk.XML.stretch(),
			rk.XML.frame(
				clust.m.spin.numcl,
				clust.plotm.cbox.plot,
				label="Advanced options"),
			save.results.m
		)
	)

tab.m.plot <- rk.XML.row(
		rk.XML.col(
			clust.plotm.type,
			rk.XML.stretch(),
			clust.plotm.preview)
	)

clust.m.full.dialog <- rk.XML.dialog(
	rk.XML.tabbook(tabs=list(
		"Data"=tab.m.data,	
		"Plot"=tab.m.plot
	)),
	label="Cluster analysis: Model based")

## logic section
	lgc.sect.m <- rk.XML.logic(
		rk.XML.connect(governor=clust.plotm.cbox.plot, client=tab.m.plot, set="enabled")
	)

## JavaScript
clust.m.js.calc <- rk.paste.JS(
	ite(clust.pre.omitNA, echo("\t# Listwise removal of missings\n\t",
		clust.m.var.data, " <- na.omit(", clust.m.var.data, ")\n")),
	ite(clust.pre.scale, echo("\t# Standardizing values\n\t",
		clust.m.var.data, " <- scale(", clust.m.var.data, ")\n")),
	echo("\t# Model based CA\n\tclust.m.result <- Mclust(data=", clust.m.var.data),
	ite(id(clust.m.spin.numcl, " != 9"), echo(",\n\t\tG=1:", clust.m.spin.numcl, "\n\t")),
	echo(")\n\n")
)

clust.m.js.plot <- rk.paste.JS(
	ite(clust.plotm.cbox.plot, rk.paste.JS(
			ite("full", echo("rk.graph.on()\n")),
			echo("\ttry({\n\t\tplot(clust.m.result,\n\t\t\tdata=",clust.m.var.data,
			",\n\t\t\twhat=\"", clust.plotm.type, "\""),
			echo(")\n\t})\n"),
			ite("full", echo("rk.graph.off()\n")), level=1)),
		echo("\trk.print(clust.m.result)\n")
)

## make a whole component
clust.m.component <- rk.plugin.component("Model based CA",
	xml=list(
		dialog=clust.m.full.dialog,
		logic=lgc.sect.m),
	js=list(
		require="mclust",
		calculate=clust.m.js.calc,
		doPrintout=clust.m.js.plot
	),
	hierarchy=list("analysis", "Cluster analysis"),
	create=c("xml", "js"))


#############
## number of clusters
#############

clust.num.var.select <- rk.XML.varselector(label="Select data")
clust.num.var.data <- rk.XML.varslot(label="Data (matrix or data.frame)", source=clust.num.var.select, classes=c("data.frame", "matrix"), required=TRUE)

clust.num.radio.type <- rk.XML.radio("Method", options=list(
		"K-means total within sum of sqares"=c(val="kmeans"),
		"Hiearchical clustering criterion (Inverse Scree)"=c(val="hclust", chk=TRUE)
	))

clust.num.spin.numcl <- rk.XML.spinbox(label="Maximum number of clusters to exexamine", min=2, initial=15, real=FALSE)

clust.num.preview <- rk.XML.preview()

clust.num.full.dialog <- rk.XML.dialog(
	rk.XML.row(
		clust.num.var.select,
		rk.XML.col(
			clust.num.var.data,
			clust.pre.frame,
			rk.XML.stretch(),
			rk.XML.frame(clust.num.radio.type, clust.num.spin.numcl),
			clust.num.frm.dist <- rk.XML.frame(clust.h.drop.dist, clust.h.spin.pwmink, label="Distance matrix"),
			clust.num.frm.clst <- rk.XML.frame(clust.h.drop.clst, label="Clustering"),
			clust.num.preview)
	)
, label="Cluster analysis: Determine number of clusters")

## logic section
	lgc.sect.num <- rk.XML.logic(
		CA.gov.dist.num <- rk.XML.convert(sources=list(string=clust.h.drop.dist), mode=c(equals="minkowski")),
		rk.XML.connect(governor=CA.gov.dist.num, client=clust.h.spin.pwmink, set="enabled"),
		CA.gov.dist.num.type <- rk.XML.convert(sources=list(string=clust.num.radio.type), mode=c(equals="hclust")),
		rk.XML.connect(governor=CA.gov.dist.num.type, client=clust.num.frm.dist, set="enabled"),
		rk.XML.connect(governor=CA.gov.dist.num.type, client=clust.num.frm.clst, set="enabled")
	)


## JavaScript
# plot of within groups sum of squares x number of clusters
# see http://www.statmethods.net/advstats/cluster.html
clust.num.js.calc <- rk.paste.JS(
	ite(clust.pre.omitNA, echo("\t# Listwise removal of missings\n\t",
		clust.num.var.data, " <- na.omit(", clust.num.var.data, ")\n")),
	ite(clust.pre.scale, echo("\t# Standardizing values\n\t",
		clust.num.var.data, " <- scale(", clust.num.var.data, ")\n")),
	ite(id(clust.num.radio.type, " == \"kmeans\" && ", clust.num.var.data), echo("\t# Calculate within groups sum of squares",
		"\n\tclust.wss <- (nrow(",clust.num.var.data,")-1) * sum(apply(",clust.num.var.data,", 2, var))\n",
		"\tfor (i in 2:",clust.num.spin.numcl,"){\n\t\tclust.wss[i] <- kmeans(",clust.num.var.data,", centers=i)$tot.withinss\n\t}\n\n")),
	ite(id(clust.num.radio.type, " == \"hclust\" && ", clust.num.var.data), echo("\t# Get clustering criterion",
		"\n\tclust.from <- nrow(",clust.num.var.data,")-",clust.num.spin.numcl,
		"\n\tclust.to <- nrow(",clust.num.var.data,")-1",
		"\n\tclust.wss <- hclust(dist(",clust.num.var.data,", method=\"", clust.h.drop.dist, "\"), method=\"",clust.h.drop.clst,"\")$height[clust.from:clust.to]\n\n"))
)

clust.num.js.print <- rk.paste.JS(
	ite("full", echo("rk.graph.on()\n")),
	echo("\ttry({\n\t\tplot(\n\t\t\t"),
	ite(id(clust.num.radio.type, " == \"kmeans\""), echo(
		"1:",clust.num.spin.numcl,",\n\t\t\tclust.wss,\n\t\t\ttype=\"b\",\n\t\t\txlab=\"Number of Clusters\",\n\t\t\tylab=\"Within groups sum of squares\",",
		"\n\t\t\tmain=\"Within sum of squares by clusters\",\n\t\t\tsub=\"Examined ", clust.num.spin.numcl, " clusters using k-means partitioning\")")),
	ite(id(clust.num.radio.type, " == \"hclust\""), echo(
		"clust.wss,\n\t\t\ttype=\"b\",\n\t\t\txlab=\"Number of Clusters\",\n\t\t\tylab=\"Agglomeration criterion\",\n\t\t\txaxt=\"n\",",
		"\n\t\t\tmain=\"Inverse Scree plot\",\n\t\t\tsub=\"Examined ", clust.num.spin.numcl, " clusters (dist: ", clust.h.drop.dist, ", hclust: ",clust.h.drop.clst,")\")",
		"\n\t\taxis(1, at=1:",clust.num.spin.numcl,", labels=",clust.num.spin.numcl, ":1)")),
	echo("\n\t})\n"),
 	ite("full", echo("rk.graph.off()\n"))
)

## make a whole component
clust.num.component <- rk.plugin.component("Determine number of clusters",
	xml=list(
		dialog=clust.num.full.dialog,
		logic=lgc.sect.num),
	js=list(
#		require="fcp",
		calculate=clust.num.js.calc,
		doPrintout=clust.num.js.print),
	hierarchy=list("plots", "Cluster analysis"),
	create=c("xml", "js"))


#############
## if you run the following function call, files will be written to tempdir!
#############
# this is where it get's serious, that is, here all of the above is put together into one plugin

cluster.plugin.dir <<- rk.plugin.skeleton(
	about.info,
	path=output.dir,
	xml=list(
		dialog=clust.k.full.dialog,
		logic=lgc.sect.k),
	js=list(results.header="Cluster analysis",
#		require="fpc",
		calculate=clust.k.js.calc,
		doPrintout=clust.k.js.plot),
	pluginmap=list(name="K-means partitioning", hierarchy=list("analysis", "Cluster analysis")),
	components=list(
		clust.h.component,
		clust.m.component,
		clust.num.component),
	create=c("pmap", "xml", "js", "desc"),
	overwrite=overwrite,
	tests=FALSE,
#	edit=TRUE,
	load=TRUE)#,
#	show=TRUE)
})